@@ -1,36 +1,99 @@
 #' @title Evolutionary rates computation along phylogenies
-#' @description The function \code{RRphylo} (\cite{Castiglione et al. 2018}) performs the phylogenetic ridge regression. It takes a tree and a vector of tip data (phenotypes) as entries, calculates the regularization factor, produces the matrices of tip to root (\code{\link{makeL}}), and node to root distances (\code{\link{makeL1}}), the vector of ancestral state estimates, the vector of predicted phenotypes, and the rates vector for all the branches of the tree. For multivariate data, rates are given as both one vector per variable, and as a multivariate vector obtained by computing the Euclidean Norm of individual rate vectors.
-#' @usage RRphylo(tree,y,cov=NULL,rootV=NULL,aces=NULL,x1=NULL,aces.x1=NULL,clus=0.5)
-#' @param tree a phylogenetic tree. The tree needs not to be ultrametric or fully dichotomous.
-#' @param y either a single vector variable or a multivariate dataset of class \sQuote{matrix}. In any case, \code{y} must be named.
-#' @param cov the covariate to be indicated if its effect on the rates must be accounted for. In this case residuals of the covariate versus the rates are used as rates. \code{'cov'} must be as long as the number of nodes plus the number of tips of the tree, which can be obtained by running \code{RRphylo} on the covariate as well, and taking the vector of ancestral states and tip values to form the covariate, as in the example below.
-#' @param rootV phenotypic value (values if multivariate) at the tree root. If \code{rootV=NULL} the function 'learns' about the root value from the 10\% tips being closest in time to the tree root, weighted by their temporal distance from the root itself (close tips phenotypes weigh more than more distant tips).
-#' @param aces a named vector (or matrix if \code{y} is multivariate) of ancestral character values at nodes. Names correspond to the nodes in the tree.
-#' @param x1 the additional predictor to be indicated to perform the multiple version of \code{RRphylo}. \code{'x1'} vector must be as long as the number of nodes plus the number of tips of the tree, which can be obtained by running \code{RRphylo} on the predictor as well, and taking the vector of ancestral states and tip values to form the \code{x1}.
-#' @param aces.x1 a named vector of ancestral character values at nodes for \code{x1}. It must be indicated if both \code{aces} and \code{x1} are specified. Names correspond to the nodes in the tree.
-#' @param clus the proportion of clusters to be used in parallel computing (only if \code{y} is multivariate).
+#' @description The function \code{RRphylo} (\cite{Castiglione et al. 2018})
+#'   performs the phylogenetic ridge regression. It takes a tree and a vector of
+#'   tip data (phenotypes) as entries, calculates the regularization factor,
+#'   produces the matrices of tip to root (\code{\link{makeL}}), and node to
+#'   root distances (\code{\link{makeL1}}), the vector of ancestral state
+#'   estimates, the vector of predicted phenotypes, and the rates vector for all
+#'   the branches of the tree. For multivariate data, rates are given as both
+#'   one vector per variable, and as a multivariate vector obtained by computing
+#'   the Euclidean Norm of individual rate vectors.
+#' @usage
+#'   RRphylo(tree,y,cov=NULL,rootV=NULL,aces=NULL,x1=NULL,aces.x1=NULL,clus=0.5)
+#' @param tree a phylogenetic tree. The tree needs not to be ultrametric or
+#'   fully dichotomous.
+#' @param y either a single vector variable or a multivariate dataset of class
+#'   \sQuote{matrix}. In any case, \code{y} must be named.
+#' @param cov the covariate to be indicated if its effect on the rates must be
+#'   accounted for. In this case residuals of the covariate versus the rates are
+#'   used as rates. \code{'cov'} must be as long as the number of nodes plus the
+#'   number of tips of the tree, which can be obtained by running \code{RRphylo}
+#'   on the covariate as well, and taking the vector of ancestral states and tip
+#'   values to form the covariate, as in the example below.
+#' @param rootV phenotypic value (values if multivariate) at the tree root. If
+#'   \code{rootV=NULL} the function 'learns' about the root value from the 10\%
+#'   tips being closest in time to the tree root, weighted by their temporal
+#'   distance from the root itself (close tips phenotypes weigh more than more
+#'   distant tips).
+#' @param aces a named vector (or matrix if \code{y} is multivariate) of
+#'   ancestral character values at nodes. Names correspond to the nodes in the
+#'   tree.
+#' @param x1 the additional predictor to be indicated to perform the multiple
+#'   version of \code{RRphylo}. \code{'x1'} vector must be as long as the number
+#'   of nodes plus the number of tips of the tree, which can be obtained by
+#'   running \code{RRphylo} on the predictor as well, and taking the vector of
+#'   ancestral states and tip values to form the \code{x1}.
+#' @param aces.x1 a named vector of ancestral character values at nodes for
+#'   \code{x1}. It must be indicated if both \code{aces} and \code{x1} are
+#'   specified. Names correspond to the nodes in the tree.
+#' @param clus the proportion of clusters to be used in parallel computing (only
+#'   if \code{y} is multivariate). To run the single-threaded version of
+#'   \code{RRphylo} set \code{clus} = 0.
 #' @export
-#' @importFrom ape multi2di Ntip is.binary.tree Nnode dist.nodes drop.tip subtrees nodelabels
+#' @importFrom ape multi2di Ntip is.binary.tree Nnode dist.nodes drop.tip
+#'   subtrees nodelabels
 #' @importFrom stats dist lm residuals weighted.mean
 #' @importFrom stats4 mle
 #' @importFrom geiger treedata tips ratematrix
 #' @importFrom phytools bind.tip
-#' @return \strong{tree} the tree used by \code{RRphylo}. The fully dichotomous version of the tree argument. For trees with polytomies, the tree is resolved by using \code{multi2di} function in the package \pkg{ape}. If the latter is a dichotomous tree, the two trees will be identical.
-#' @return \strong{tip.path} a \eqn{n * m} matrix, where n=number of tips and m=number of branches (i.e. 2*n-1). Each row represents the branch lengths along a root-to-tip path.
-#' @return \strong{node.path} a \eqn{n * n} matrix, where n=number of internal branches. Each row represents the branch lengths along a root-to-node path.
-#' @return \strong{rates} single rate values computed for each branch of the tree. If \code{y} is a single vector variable, rates are equal to multiple.rates. If \code{y} is a multivariate dataset, rates are computed as the square root of the sum of squares of each row of \code{$multiple.rates}.
+#' @return \strong{tree} the tree used by \code{RRphylo}. The fully dichotomous
+#'   version of the tree argument. For trees with polytomies, the tree is
+#'   resolved by using \code{multi2di} function in the package \pkg{ape}. If the
+#'   latter is a dichotomous tree, the two trees will be identical.
+#' @return \strong{tip.path} a \eqn{n * m} matrix, where n=number of tips and
+#'   m=number of branches (i.e. 2*n-1). Each row represents the branch lengths
+#'   along a root-to-tip path.
+#' @return \strong{node.path} a \eqn{n * n} matrix, where n=number of internal
+#'   branches. Each row represents the branch lengths along a root-to-node path.
+#' @return \strong{rates} single rate values computed for each branch of the
+#'   tree. If \code{y} is a single vector variable, rates are equal to
+#'   multiple.rates. If \code{y} is a multivariate dataset, rates are computed
+#'   as the square root of the sum of squares of each row of
+#'   \code{$multiple.rates}.
 #' @return \strong{aces} the phenotypes reconstructed at nodes.
-#' @return \strong{predicted.phenotypes} the vector of estimated tip values. It is a matrix in the case of multivariate data.
-#' @return \strong{multiple.rates} a \eqn{n * m} matrix, where n= number of branches (i.e. n*2-1) and m = number of variables. For each branch, the column entries represent the evolutionary rate.
-#' @return \strong{lambda} the regularization factor fitted within \code{RRphylo} by the inner function \code{optL}. With multivariate data, several \code{optL} runs are performed. Hence, the function provides a single lambda for each individual variable.
-#' @return \strong{ace.values} if \code{aces} are specified, the function returns a dataframe containing the corresponding node number on the \code{RRphylo} tree  for each node , along with estimated values.
-#' @return \strong{x1.rate} if \code{x1} is specified, the function returns the partial regression coefficient for \code{x1}.
-#' @author Pasquale Raia, Silvia Castiglione, Carmela Serio, Alessandro Mondanaro, Marina Melchionna, Mirko Di Febbraro, Antonio Profico, Francesco Carotenuto
-#' @references Castiglione, S., Tesone, G., Piccolo, M., Melchionna, M., Mondanaro, A., Serio, C., Di Febbraro, M., & Raia, P.(2018). A new method for testing evolutionary rate variation and shifts in phenotypic evolution. \emph{Methods in Ecology and Evolution}, 9: 974-983.doi:10.1111/2041-210X.12954
-#' @references Serio, C., Castiglione, S., Tesone, G., Piccolo, M., Melchionna, M., Mondanaro, A., Di Febbraro, M., & Raia, P.(2019). Macroevolution of toothed whales exceptional relative brain size. \emph{Evolutionary Biology}, 46: 332-342. doi:10.1007/s11692-019-09485-7
-#' @references  Melchionna, M., Mondanaro, A., Serio, C., Castiglione, S., Di Febbraro, M., Rook, L.,Diniz-Filho,J.A.F., Manzi, G., Profico, A., Sansalone, G., & Raia, P.(2020).Macroevolutionary trends of brain mass in Primates. \emph{Biological Journal of the Linnean Society}, 129: 14-25. doi:10.1093/biolinnean/blz161
+#' @return \strong{predicted.phenotypes} the vector of estimated tip values. It
+#'   is a matrix in the case of multivariate data.
+#' @return \strong{multiple.rates} a \eqn{n * m} matrix, where n= number of
+#'   branches (i.e. n*2-1) and m = number of variables. For each branch, the
+#'   column entries represent the evolutionary rate.
+#' @return \strong{lambda} the regularization factor fitted within
+#'   \code{RRphylo} by the inner function \code{optL}. With multivariate data,
+#'   several \code{optL} runs are performed. Hence, the function provides a
+#'   single lambda for each individual variable.
+#' @return \strong{ace.values} if \code{aces} are specified, the function
+#'   returns a dataframe containing the corresponding node number on the
+#'   \code{RRphylo} tree  for each node , along with estimated values.
+#' @return \strong{x1.rate} if \code{x1} is specified, the function returns the
+#'   partial regression coefficient for \code{x1}.
+#' @author Pasquale Raia, Silvia Castiglione, Carmela Serio, Alessandro
+#'   Mondanaro, Marina Melchionna, Mirko Di Febbraro, Antonio Profico, Francesco
+#'   Carotenuto
+#' @references Castiglione, S., Tesone, G., Piccolo, M., Melchionna, M.,
+#'   Mondanaro, A., Serio, C., Di Febbraro, M., & Raia, P.(2018). A new method
+#'   for testing evolutionary rate variation and shifts in phenotypic evolution.
+#'   \emph{Methods in Ecology and Evolution}, 9:
+#'   974-983.doi:10.1111/2041-210X.12954
+#' @references Serio, C., Castiglione, S., Tesone, G., Piccolo, M., Melchionna,
+#'   M., Mondanaro, A., Di Febbraro, M., & Raia, P.(2019). Macroevolution of
+#'   toothed whales exceptional relative brain size. \emph{Evolutionary
+#'   Biology}, 46: 332-342. doi:10.1007/s11692-019-09485-7
+#' @references  Melchionna, M., Mondanaro, A., Serio, C., Castiglione, S., Di
+#'   Febbraro, M., Rook, L.,Diniz-Filho,J.A.F., Manzi, G., Profico, A.,
+#'   Sansalone, G., & Raia, P.(2020).Macroevolutionary trends of brain mass in
+#'   Primates. \emph{Biological Journal of the Linnean Society}, 129: 14-25.
+#'   doi:10.1093/biolinnean/blz161
 #' @examples
-#'  \donttest{
+#'  \dontrun{
 #' data("DataOrnithodirans")
 #' DataOrnithodirans$treedino->treedino
 #' DataOrnithodirans$massdino->massdino
@@ -196,9 +259,10 @@ RRphylo<-function (tree, y, cov = NULL, rootV = NULL, aces = NULL,x1=NULL,aces.x
       i = 1
       while (i <= length(N)) {
         nn <- getMRCA(treeN, tar.tips[[i]])
-        treeN <- bind.tip(treeN, tip.label = paste("nod",
-                                                   N[i], sep = ""), edge.length = 0, where = nn,
-                          position = 0.001)
+        treeN$edge.length[which(treeN$edge[,2]==nn)]->edlen
+        if(edlen<=0.001) edlen/10->pp else 0.001->pp
+        treeN <- bind.tip(treeN, tip.label = paste("nod",N[i], sep = ""),
+                          edge.length = 0, where = nn,position = edlen)
         npos <- which(treeN$tip.label == paste("nod", N[i], sep = ""))
         if (npos == 1) ynew <- rbind(P[i, ], ynew)
         if (npos == nrow(ynew) + 1) ynew <- rbind(ynew, P[i, ]) else {
@@ -265,12 +329,11 @@ RRphylo<-function (tree, y, cov = NULL, rootV = NULL, aces = NULL,x1=NULL,aces.x
       i = 1
       while (i <= length(N)) {
         nn <- getMRCA(treeN, tar.tips[[i]])
-        treeN <- bind.tip(treeN,
-                          tip.label = paste("nod",N[i], sep = ""),
-                          edge.length = 0, where = nn,position = 0.001)
-        npos <- which(treeN$tip.label ==
-                        paste("nod",N[i], sep = ""))
-
+        treeN$edge.length[which(treeN$edge[,2]==nn)]->edlen
+        if(edlen<=0.001) edlen/10->pp else 0.001->pp
+        treeN <- bind.tip(treeN, tip.label = paste("nod",N[i], sep = ""),
+                          edge.length = 0, where = nn,position = edlen)
+        npos <- which(treeN$tip.label == paste("nod", N[i], sep = ""))
         if (npos == 1) ynew <- c(P[i], ynew)
 
         if (npos == length(ynew) + 1) ynew <- c(ynew, P[i]) else {
@@ -325,7 +388,7 @@ RRphylo<-function (tree, y, cov = NULL, rootV = NULL, aces = NULL,x1=NULL,aces.x
     y.real <- y
     rv.real <- rootV
     res <- list()
-    cl <- makeCluster(round((detectCores() * clus), 0))
+    if(round((detectCores() * clus), 0)==0) cl<-makeCluster(1) else cl <- makeCluster(round((detectCores() * clus), 0))
     registerDoParallel(cl)
     res <- foreach(i = 1:k, .packages = c("stats4", "ape")) %dopar% {
       #for(i in 1:k){
@@ -364,6 +427,8 @@ RRphylo<-function (tree, y, cov = NULL, rootV = NULL, aces = NULL,x1=NULL,aces.x
       }
     }
     stopCluster(cl)
+
+
     aceRR <- do.call(cbind, lapply(res, "[[", 1))
     betas <- do.call(cbind, lapply(res, "[[", 2))
     y.hat <- do.call(cbind, lapply(res, "[[", 3))
@@ -481,8 +546,7 @@ RRphylo<-function (tree, y, cov = NULL, rootV = NULL, aces = NULL,x1=NULL,aces.x
         factOut <- which(betas != "0")
         betas[factOut] <- res
         betas[zeroes] <- 0
-      }
-      else {
+      }else {
         R <- log(abs(betas))
         Y <- abs(cov)
         res <- residuals(lm(R ~ Y))
