@@ -1,35 +1,116 @@
 #' @title Testing RRphylo methods overfit
-#' @description Testing the robustness of \code{\link{search.trend}} (\cite{Castiglione et al. 2019a}), \code{\link{search.shift}} (\cite{Castiglione et al. 2018}), and  \code{\link{search.conv}} (\cite{Castiglione et al. 2019b}) results to sampling effects and phylogenetic uncertainty.
-#' @usage overfitRR(RR,y,s=0.25,swap.args=NULL,trend.args=NULL,shift.args=NULL,conv.args=NULL,
-#' aces=NULL,x1=NULL,aces.x1=NULL,cov=NULL,rootV=NULL,nsim=100,clus=.5)
+#' @description Testing the robustness of \code{\link{search.trend}}
+#'   (\cite{Castiglione et al. 2019a}), \code{\link{search.shift}}
+#'   (\cite{Castiglione et al. 2018}), and  \code{\link{search.conv}}
+#'   (\cite{Castiglione et al. 2019b}) results to sampling effects and
+#'   phylogenetic uncertainty.
+#' @usage
+#'   overfitRR(RR,y,s=0.25,swap.args=NULL,trend.args=NULL,shift.args=NULL,conv.args=NULL,
+#'    aces=NULL,x1=NULL,aces.x1=NULL,cov=NULL,rootV=NULL,nsim=100,clus=.5)
 #' @param RR an object produced by \code{\link{RRphylo}}.
 #' @param y a named vector of phenotypes.
 #' @param s the percentage of tips to be cut off. It is set at 25\% by default.
-#' @param swap.args a list of arguments to be passed to the function \code{\link{swapONE}}, including \code{list(si=NULL,si2=NULL,} \code{node=NULL)}. If \code{swap.arg} is unspecified, the function automatically sets both \code{si} and \code{si2} to 0.1.
-#' @param trend.args a list of arguments specific to the function \code{search.trend}, including \code{list(node=NULL)}. If a trend for the whole tree is to be tested, type \code{trend.args = list()}. No trend is tested if left unspecified.
-#' @param shift.args a list of arguments specific to the function \code{search.shift}, including \code{list(node=NULL,} \code{state=NULL)}. Arguments \code{node} and \code{state} can be specified at the same time.
-#' @param conv.args a list of arguments specific to the function \code{search.conv}, including \code{list(node=NULL,} \code{state=NULL, declust=FALSE)}. Arguments \code{node} and \code{state} can be specified at the same time.
-#' @param aces if used to produce the \code{RR} object, the vector of those ancestral character values at nodes known in advance must be specified. Names correspond to the nodes in the tree.
-#' @param x1 the additional predictor to be specified if the RR object has been created using an additional predictor (i.e. multiple version of \code{RRphylo}). \code{'x1'} vector must be as long as the number of nodes plus the number of tips of the tree, which can be obtained by running \code{RRphylo} on the predictor as well, and taking the vector of ancestral states and tip values to form the \code{x1}.
-#' @param aces.x1 a named vector of ancestral character values at nodes for \code{x1}. It must be indicated if the RR object has been created using both \code{aces} and \code{x1}. Names correspond to the nodes in the tree.
-#' @param cov if used to produce the \code{RR} object, the covariate must be specified. As in \code{RRphylo}, the covariate vector must be as long as the number of nodes plus the number of tips of the tree, which can be obtained by running \code{RRphylo} on the covariate as well, and taking the vector of ancestral states and tip values to form the covariate.
-#' @param rootV if used to produce the \code{RR} object, the phenotypic value at the tree root must be specified.
-#' @param nsim number of simulations to be performed. It is set at 100 by default.
-#' @param clus the proportion of clusters to be used in parallel computing.
+#' @param swap.args a list of arguments to be passed to the function
+#'   \code{\link{swapONE}}, including \code{list(si=NULL,si2=NULL,}
+#'   \code{node=NULL)}. If \code{swap.arg} is unspecified, the function
+#'   automatically sets both \code{si} and \code{si2} to 0.1.
+#' @param trend.args a list of arguments specific to the function
+#'   \code{search.trend}, including \code{list(node=NULL)}. If a trend for the
+#'   whole tree is to be tested, type \code{trend.args = list()}. No trend is
+#'   tested if left unspecified.
+#' @param shift.args a list of arguments specific to the function
+#'   \code{search.shift}, including \code{list(node=NULL,} \code{state=NULL)}.
+#'   Arguments \code{node} and \code{state} can be specified at the same time.
+#' @param conv.args a list of arguments specific to the function
+#'   \code{search.conv}, including \code{list(node=NULL,} \code{state=NULL,
+#'   declust=FALSE)}. Arguments \code{node} and \code{state} can be specified at
+#'   the same time.
+#' @param aces if used to produce the \code{RR} object, the vector of those
+#'   ancestral character values at nodes known in advance must be specified.
+#'   Names correspond to the nodes in the tree.
+#' @param x1 the additional predictor to be specified if the RR object has been
+#'   created using an additional predictor (i.e. multiple version of
+#'   \code{RRphylo}). \code{'x1'} vector must be as long as the number of nodes
+#'   plus the number of tips of the tree, which can be obtained by running
+#'   \code{RRphylo} on the predictor as well, and taking the vector of ancestral
+#'   states and tip values to form the \code{x1}.
+#' @param aces.x1 a named vector of ancestral character values at nodes for
+#'   \code{x1}. It must be indicated if the RR object has been created using
+#'   both \code{aces} and \code{x1}. Names correspond to the nodes in the tree.
+#' @param cov if used to produce the \code{RR} object, the covariate must be
+#'   specified. As in \code{RRphylo}, the covariate vector must be as long as
+#'   the number of nodes plus the number of tips of the tree, which can be
+#'   obtained by running \code{RRphylo} on the covariate as well, and taking the
+#'   vector of ancestral states and tip values to form the covariate.
+#' @param rootV if used to produce the \code{RR} object, the phenotypic value at
+#'   the tree root must be specified.
+#' @param nsim number of simulations to be performed. It is set at 100 by
+#'   default.
+#' @param clus the proportion of clusters to be used in parallel computing. To
+#'   run the single-threaded version of \code{overfitRR} set \code{clus} = 0.
 #' @return The function returns a 'list' containing:
+#' @return \strong{$mean.sampling} the mean proportion of species actually
+#'   removed from the tree over the iterations.
 #' @return \strong{$rootCI} the 95\% confidence interval around the root value.
-#' @return \strong{$ace.regressions} the results of linear regression between ancestral state estimates before and after the subsampling.
-#' @return \strong{$conv.results} a list including results for \code{search.conv} performed under \code{clade} and \code{state} conditions. If a node pair is specified within \code{conv.args}, the \code{$clade} object contains the percentage of simulations producing significant p-values for convergence between the clades. If a state vector is supplied within \code{conv.args}, the object \code{$state} contains the percentage of simulations producing significant p-values for convergence within (single state) or between states (multiple states).
-#' @return \strong{$shift.results} a list including results for \code{search.shift} performed under \code{clade} and \code{sparse} conditions. If one or more nodes are specified within \code{shift.args}, the \code{$clade} object contains for each node the percentage of simulations producing significant p-value separated by shift sign, and the same figures by considering all the specified nodes as evolving under a single rate (all.clades).If a state vector is supplied within \code{shift.args}, the object \code{$sparse} contains the percentage of simulations producing significant p-value separated by shift sign ($p.states).
-#' @return \strong{$trend.results} a list including the percentage of simulations showing significant p-values for phenotypes versus age and absolute rates versus age regressions for the entire tree separated by slope sign ($tree). If one or more nodes are specified within \code{trend.args}, the list also includes the same results at nodes ($node) and the results for comparison between nodes ($comparison).
+#' @return \strong{$ace.regressions} the results of linear regression between
+#'   ancestral state estimates before and after the subsampling.
+#' @return \strong{$conv.results} a list including results for
+#'   \code{search.conv} performed under \code{clade} and \code{state}
+#'   conditions. If a node pair is specified within \code{conv.args}, the
+#'   \code{$clade} object contains the percentage of simulations producing
+#'   significant p-values for convergence between the clades. If a state vector
+#'   is supplied within \code{conv.args}, the object \code{$state} contains the
+#'   percentage of simulations producing significant p-values for convergence
+#'   within (single state) or between states (multiple states).
+#' @return \strong{$shift.results} a list including results for
+#'   \code{search.shift} performed under \code{clade} and \code{sparse}
+#'   conditions. If one or more nodes are specified within \code{shift.args},
+#'   the \code{$clade} object contains for each node the percentage of
+#'   simulations producing significant p-value separated by shift sign, and the
+#'   same figures by considering all the specified nodes as evolving under a
+#'   single rate (all.clades).If a state vector is supplied within
+#'   \code{shift.args}, the object \code{$sparse} contains the percentage of
+#'   simulations producing significant p-value separated by shift sign
+#'   ($p.states).
+#' @return \strong{$trend.results} a list including the percentage of
+#'   simulations showing significant p-values for phenotypes versus age and
+#'   absolute rates versus age regressions for the entire tree separated by
+#'   slope sign ($tree). If one or more nodes are specified within
+#'   \code{trend.args}, the list also includes the same results at nodes ($node)
+#'   and the results for comparison between nodes ($comparison).
 #' @author Silvia Castiglione, Carmela Serio, Pasquale Raia
-#' @details Methods using a large number of parameters risk being overfit. This usually translates in poor fitting with data and trees other than the those originally used. With \code{RRphylo} methods this risk is usually very low. However, the user can assess how robust the results got by applying \code{search.shift}, \code{search.trend} or \code{search.conv} are by running \code{overfitRR}. With the latter, the original tree and data are subsampled by specifying a \code{s} parameter, that is the proportion of tips to be removed from the tree. Internally, \code{overfitRR} further shuffles the tree by using the function \code{\link{swapONE}}. Thereby, both the potential for overfit and phylogenetic uncertainty are accounted for straight away.
+#' @details Methods using a large number of parameters risk being overfit. This
+#'   usually translates in poor fitting with data and trees other than the those
+#'   originally used. With \code{RRphylo} methods this risk is usually very low.
+#'   However, the user can assess how robust the results got by applying
+#'   \code{search.shift}, \code{search.trend} or \code{search.conv} are by
+#'   running \code{overfitRR}. With the latter, the original tree and data are
+#'   subsampled by specifying a \code{s} parameter, that is the proportion of
+#'   tips to be removed from the tree. In some cases, though, removing as many
+#'   tips as imposed by \code{s} would delete too many tips right in clades
+#'   and/or states under testing. In these cases, the function maintains no less
+#'   than 5 species at least in each clade/state under testing (or all species
+#'   if there is less), reducing the sampling parameter \code{s} if necessary.
+#'   Internally, \code{overfitRR} further shuffles the tree by using the
+#'   function \code{\link{swapONE}}. Thereby, both the potential for overfit and
+#'   phylogenetic uncertainty are accounted for straight away.
 #' @export
-#' @importFrom rlist list.append
-#' @importFrom ddpcr quiet
-#' @references Castiglione, S., Tesone, G., Piccolo, M., Melchionna, M., Mondanaro, A., Serio, C., Di Febbraro, M., & Raia, P. (2018). A new method for testing evolutionary rate variation and shifts in phenotypic evolution. \emph{Methods in Ecology and Evolution}, 9: 974-983.doi:10.1111/2041-210X.12954
-#' @references Castiglione, S., Serio, C., Mondanaro, A., Di Febbraro, M., Profico, A., Girardi, G., & Raia, P. (2019a) Simultaneous detection of macroevolutionary patterns in phenotypic means and rate of change with and within phylogenetic trees including extinct species. \emph{PLoS ONE}, 14: e0210101. https://doi.org/10.1371/journal.pone.0210101
-#' @references Castiglione, S., Serio, C., Tamagnini, D., Melchionna, M., Mondanaro, A., Di Febbraro, M., Profico, A., Piras, P.,Barattolo, F., & Raia, P. (2019b). A new, fast method to search for morphological convergence with shape data. \emph{PLoS ONE}, 14, e0226949. https://doi.org/10.1371/journal.pone.0226949
+#' @importFrom utils setTxtProgressBar txtProgressBar
+#' @references Castiglione, S., Tesone, G., Piccolo, M., Melchionna, M.,
+#'   Mondanaro, A., Serio, C., Di Febbraro, M., & Raia, P. (2018). A new method
+#'   for testing evolutionary rate variation and shifts in phenotypic evolution.
+#'   \emph{Methods in Ecology and Evolution}, 9:
+#'   974-983.doi:10.1111/2041-210X.12954
+#' @references Castiglione, S., Serio, C., Mondanaro, A., Di Febbraro, M.,
+#'   Profico, A., Girardi, G., & Raia, P. (2019a) Simultaneous detection of
+#'   macroevolutionary patterns in phenotypic means and rate of change with and
+#'   within phylogenetic trees including extinct species. \emph{PLoS ONE}, 14:
+#'   e0210101. https://doi.org/10.1371/journal.pone.0210101
+#' @references Castiglione, S., Serio, C., Tamagnini, D., Melchionna, M.,
+#'   Mondanaro, A., Di Febbraro, M., Profico, A., Piras, P.,Barattolo, F., &
+#'   Raia, P. (2019b). A new, fast method to search for morphological
+#'   convergence with shape data. \emph{PLoS ONE}, 14, e0226949.
+#'   https://doi.org/10.1371/journal.pone.0226949
 #' @examples
 #' \dontrun{
 #' data("DataOrnithodirans")
@@ -40,7 +121,7 @@
 #'
 #' # Extract Pterosaurs tree and data
 #' library(ape)
-#' extract.clade(treedino,748)->treeptero
+#' extract.clade(treedino,746)->treeptero
 #' massdino[match(treeptero$tip.label,names(massdino))]->massptero
 #' massptero[match(treeptero$tip.label,names(massptero))]->massptero
 #'
@@ -120,6 +201,11 @@ overfitRR<-function(RR,y,
   # require(ddpcr)
   # require(rlist)
 
+  if (!requireNamespace("ddpcr", quietly = TRUE)) {
+    stop("Package \"ddpcr\" needed for this function to work. Please install it.",
+         call. = FALSE)
+  }
+
   RR$tree->tree
 
   # if (inherits(y,"data.frame"))
@@ -127,6 +213,10 @@ overfitRR<-function(RR,y,
   if(is.null(nrow(y))) y <- treedata(tree, y, sort = TRUE)[[2]][,1] else y <- treedata(tree, y, sort = TRUE)[[2]]
 
   if (length(y) > Ntip(tree)) {
+    # if((Ntip(tree)-round(Ntip(tree)*s))<ncol(y))
+    # stop(paste("After the sampling there are more variables than observations.
+    #            Please consider running your preliminary analyses with",
+    #            (Ntip(tree)-round(Ntip(tree)*s)),"variables.",sep=" "))
     RR$aces->y.ace
     tree$node.label<-rownames(y.ace)
     y[match(tree$tip.label,rownames(y)),]->y
@@ -178,24 +268,69 @@ overfitRR<-function(RR,y,
     conv.declust<-NULL
   }
 
+
+  pb = txtProgressBar(min = 0, max = nsim, initial = 0)
+
   rootlist<-list()
   acefit<-STcut<-SScut<-SScutS<-SCcut<-SCcutS<-list()
+  real.s<-array()
   for(k in 1:nsim){
+    setTxtProgressBar(pb,k)
     '%ni%' <- Negate('%in%')
+    # repeat({
+    #   #suppressWarnings(swapONE(tree,0.1,0.1)[[1]])->tree.swap
+    #   suppressWarnings(swapONE(tree,si=si,si2=si2,node=swap.node,plot.swap=FALSE)[[1]])->tree.swap
+    #   #sample(y,round((Ntip(tree)*s),0))->offs
+    #   #sapply(trend.node,function(x) length(tips(tree,x))-length(which(names(offs)%in%tips(tree,x))))->lt
+    #   #sapply(shift.node,function(x) length(tips(tree,x))-length(which(names(offs)%in%tips(tree,x))))->lss
+    #   sample(tree$tip.label,round((Ntip(tree)*s),0))->offs
+    #   sapply(trend.node,function(x) length(tips(tree,x))-length(which(offs%in%tips(tree,x))))->lt
+    #   sapply(shift.node,function(x) length(tips(tree,x))-length(which(offs%in%tips(tree,x))))->lss
+    #   sapply(conv.node,function(x) length(tips(tree,x))-length(which(offs%in%tips(tree,x))))->lsc
+    #   if(length(which(lt<5))==0&length(which(lss<5))==0&length(which(lsc<5))==0) break
+    # })
 
+    if(s>0){
+      unlist(lapply(trend.node,function(x) {
+        length(tips(tree,x))->lenx
+        if(lenx<=5) tips(tree,x) else sample(tips(tree,x),5)
+      }))->out.st
+      unlist(lapply(shift.node,function(x) {
+        length(tips(tree,x))->lenx
+        if(lenx<=5) tips(tree,x) else sample(tips(tree,x),5)
+      }))->out.ss
+      unlist(lapply(conv.node,function(x) {
+        length(tips(tree,x))->lenx
+        if(lenx<=5) tips(tree,x) else sample(tips(tree,x),5)
+      }))->out.sc
 
-    repeat({
-      #suppressWarnings(swapONE(tree,0.1,0.1)[[1]])->tree.swap
-      suppressWarnings(swapONE(tree,si=si,si2=si2,node=swap.node,plot.swap=FALSE)[[1]])->tree.swap
-      #sample(y,round((Ntip(tree)*s),0))->offs
-      #sapply(trend.node,function(x) length(tips(tree,x))-length(which(names(offs)%in%tips(tree,x))))->lt
-      #sapply(shift.node,function(x) length(tips(tree,x))-length(which(names(offs)%in%tips(tree,x))))->lss
-      sample(tree$tip.label,round((Ntip(tree)*s),0))->offs
-      sapply(trend.node,function(x) length(tips(tree,x))-length(which(offs%in%tips(tree,x))))->lt
-      sapply(shift.node,function(x) length(tips(tree,x))-length(which(offs%in%tips(tree,x))))->lss
-      sapply(conv.node,function(x) length(tips(tree,x))-length(which(offs%in%tips(tree,x))))->lsc
-      if(length(which(lt<5))==0&length(which(lss<5))==0&length(which(lsc<5))==0) break
-    })
+      if(!is.null(shift.state)){
+        table(shift.state)->tab.ss
+        unlist(lapply(1:length(tab.ss),function(x) {
+          if(tab.ss[x]<=5) names(which(shift.state==names(tab.ss)[x])) else
+            sample(names(which(shift.state==names(tab.ss)[x])),5)
+        }))->out.st.ss
+      } else out.st.ss<-NULL
+
+      if(!is.null(conv.state)){
+        table(conv.state)->tab.cs
+        unlist(lapply(1:length(tab.cs),function(x) {
+          if(tab.cs[x]<=5) names(which(conv.state==names(tab.cs)[x])) else
+            sample(names(which(conv.state==names(tab.cs)[x])),5)
+        }))->out.st.sc
+      }else out.st.sc<-NULL
+
+      unique(c(out.st,out.ss,out.sc,out.st.ss,out.st.sc))->outs
+      if(length(outs>0)) tree$tip.label[-match(outs,tree$tip.label)]->samtips else tree$tip.label->samtips
+
+      sx<-s
+      repeat({
+        if(length(samtips)>Ntip(tree)*sx) break else s*.9->sx
+      })
+      sample(samtips,round(Ntip(tree)*sx,0))->offs
+    }
+
+    suppressWarnings(swapONE(tree,si=si,si2=si2,node=swap.node,plot.swap=FALSE)[[1]])->tree.swap
 
     if(length(y)>Ntip(tree)) y[match(tree.swap$tip.label,rownames(y)),]->y else y[match(tree.swap$tip.label,names(y))]->y
 
@@ -220,6 +355,8 @@ overfitRR<-function(RR,y,
       y.ace->y.acecut
     }
 
+    1-(Ntip(treecut)/Ntip(tree))->real.s[k]
+
     if(is.null(cov)==FALSE) {
       cov[match(c(tree.swap$node.label,tree.swap$tip.label), names(cov))]->cov
       if(length(y)>Ntip(tree))
@@ -303,11 +440,11 @@ overfitRR<-function(RR,y,
     if(is.null(rootV)==FALSE) rootV->rootVcut else rootVcut<-NULL
 
     RRphylo(treecut,ycut,aces=acescut,x1=x1cut,aces.x1=aces.x1cut,cov=covcut,rootV = rootVcut,clus=clus)->RRcut
-    if(trend|is.null(trend.node)==FALSE) quiet(search.trend(RRcut,ycut,x1=x1cut,node=trend.node.cut,foldername=tempdir(),cov=covcut,clus=clus)->stcut->STcut[[k]],all=TRUE)
-    if(is.null(shift.node)==FALSE) quiet(search.shift(RRcut,status.type="clade",node=shift.node.cut,foldername=tempdir())->sscut->SScut[[k]],all=TRUE)
-    if(is.null(shift.state)==FALSE) quiet(search.shift(RRcut,status.type="sparse",state=shift.state.cut,foldername=tempdir())->sscut->SScutS[[k]],all=TRUE)
-    if(is.null(conv.node)==FALSE) quiet(search.conv(RR=RRcut,y=ycut,nodes=conv.node.cut,aceV=acescut,clus=clus,foldername=tempdir())->sccut->SCcut[[k]],all=TRUE)
-    if(is.null(conv.state)==FALSE) quiet(search.conv(tree=treecut,y=ycut,state=conv.state.cut,aceV=acescut,declust=conv.declust,clus=clus,foldername=tempdir())->sccut->SCcutS[[k]],all=TRUE)
+    if(trend|is.null(trend.node)==FALSE) ddpcr::quiet(search.trend(RRcut,ycut,x1=x1cut,node=trend.node.cut,foldername=tempdir(),cov=covcut,clus=clus)->stcut->STcut[[k]],all=TRUE)
+    if(is.null(shift.node)==FALSE) ddpcr::quiet(search.shift(RRcut,status.type="clade",node=shift.node.cut,foldername=tempdir())->sscut->SScut[[k]],all=TRUE)
+    if(is.null(shift.state)==FALSE) ddpcr::quiet(search.shift(RRcut,status.type="sparse",state=shift.state.cut,foldername=tempdir())->sscut->SScutS[[k]],all=TRUE)
+    if(is.null(conv.node)==FALSE) ddpcr::quiet(search.conv(RR=RRcut,y=ycut,nodes=conv.node.cut,aceV=acescut,clus=clus,foldername=tempdir())->sccut->SCcut[[k]],all=TRUE)
+    if(is.null(conv.state)==FALSE) ddpcr::quiet(search.conv(tree=treecut,y=ycut,state=conv.state.cut,aceV=acescut,declust=conv.declust,clus=clus,foldername=tempdir())->sccut->SCcutS[[k]],all=TRUE)
 
     RRcut$aces[1,]->rootlist[[k]]
     summary(lm(y.acecut~RRcut$aces))->acefit[[k]]
@@ -330,7 +467,7 @@ overfitRR<-function(RR,y,
   }
 
   if(is.null(shift.node)==FALSE){
-    if(length(SScut[[1]])>2){
+    if(length(SScut[[1]])>=2){
       unlist(lapply(lapply(SScut,"[[",1),function(x) x[,2]))-> p.ran.whole
       c(length(which(p.ran.whole>=0.975))/nsim,length(which(p.ran.whole<=0.025))/nsim)->p.shift.whole
       do.call(rbind,lapply(lapply(SScut,"[[",2),function(x) x[,2]))-> p.ran
@@ -636,8 +773,8 @@ overfitRR<-function(RR,y,
   list(p.convC,p.convS)->conv.res
   names(conv.res)<-c("clade","state")
 
-  res<-list(root.conf.int,acefit,conv.res,shift.res,trend.res)
-  names(res)<-c("rootCI","ace.regressions","conv.results","shift.results","trend.results")
+  res<-list(mean(real.s),root.conf.int,acefit,conv.res,shift.res,trend.res)
+  names(res)<-c("mean.sampling","rootCI","ace.regressions","conv.results","shift.results","trend.results")
 
   return(res)
 }
